@@ -36,7 +36,6 @@
 #include <gdk-pixbuf/gdk-pixbuf.h>
 
 typedef struct _MetaGroup MetaGroup;
-typedef struct _MetaStruts MetaStruts;
 
 typedef gboolean (*MetaWindowForeachFunc) (MetaWindow *window,
                                            void       *data);
@@ -67,15 +66,6 @@ typedef enum {
   META_CLIENT_TYPE_MAX_RECOGNIZED = 2
 } MetaClientType;
 
-struct _MetaStruts
-{
-  /* struts */
-  MetaRectangle left;
-  MetaRectangle right;
-  MetaRectangle top;
-  MetaRectangle bottom;
-};
-
 struct _MetaWindow
 {
   MetaDisplay *display;
@@ -276,7 +266,7 @@ struct _MetaWindow
   guint calc_placement : 1;
 
   /* Note: can be NULL */
-  MetaStruts *struts;
+  GSList *struts;
 
   /* Transient parent is a root window */
   guint transient_parent_is_root_window : 1;
@@ -318,6 +308,9 @@ struct _MetaWindow
   /* set to the most recent user-interaction event timestamp that we
      know about for this window */
   guint32 net_wm_user_time;
+
+  /* window that gets updated net_wm_user_time values */
+  Window user_time_window;
   
   /* The size we set the window to last (i.e. what we believe
    * to be its actual size on the server). The x, y are
@@ -464,12 +457,20 @@ void        meta_window_get_position       (MetaWindow  *window,
 void        meta_window_get_user_position  (MetaWindow  *window,
                                             int         *x,
                                             int         *y);
+
+/* Gets root coords for x, y, width & height of client window; uses
+ * meta_window_get_position for x & y.
+ */
+void        meta_window_get_client_root_coords (MetaWindow    *window,
+                                                MetaRectangle *rect);
+
 /* gets position we need to set to stay in current position,
  * assuming position will be gravity-compensated. i.e.
  * this is the position a client would send in a configure
  * request.
  */
 void        meta_window_get_gravity_position (MetaWindow  *window,
+                                              int          gravity,
                                               int         *x,
                                               int         *y);
 /* Get geometry for saving in the session; x/y are gravity
@@ -509,6 +510,13 @@ void meta_window_send_icccm_message (MetaWindow *window,
                                      guint32     timestamp);
 
 
+void     meta_window_move_resize_request(MetaWindow *window,
+                                         guint       value_mask,
+                                         int         gravity,
+                                         int         x,
+                                         int         y,
+                                         int         width,
+                                         int         height);
 gboolean meta_window_configure_request (MetaWindow *window,
                                         XEvent     *event);
 gboolean meta_window_property_notify   (MetaWindow *window,
@@ -580,6 +588,7 @@ MetaWindow* meta_window_find_root_ancestor    (MetaWindow *window);
 
 void meta_window_begin_grab_op (MetaWindow *window,
                                 MetaGrabOp  op,
+                                gboolean    frame_action,
                                 guint32     timestamp);
 
 void meta_window_update_keyboard_resize (MetaWindow *window,
